/*
 * Copyright (c) 2008, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * \file 
 * 
 * Standalone executable that downloads a point cloud from the db and
 * writes it to a file
 *
 * \author Bhaskara Marthi
 */

#include <pcl/io/pcd_io.h>
#include <pcl/point_types.h>
#include <mongo_ros/message_collection.h>
#include <semanticmodel/BlobMessage.h>
#include <string>
#include <ros/ros.h>

namespace sm=semanticmodel;
namespace mr=mongo_ros;

using std::string;
using std::cout;

typedef mr::MessageWithMetadata<sm::BlobMessage>::ConstPtr BlobPtr;
typedef pcl::PointXYZRGB Point;
typedef pcl::PointCloud<Point> PointCloud;

int main (int argc, char** argv)
{
  ros::init(argc, argv, "download_cloud");
  const string filename = "foo.pcd";
  if (argc!=3) {
    cout << "Usage: download_cloud COLLECTION IND";
    exit(1);
  }
    
  mongo_ros::MessageCollection<sm::BlobMessage>
    blobs("semantic_world_model", argv[1], "bab.willowgarage.com", 27018);

  const int n = atoi(argv[2]);

  BlobPtr blob = blobs.findOne(mr::Query("id", n));
  PointCloud cloud;
  pcl::fromROSMsg(blob->points, cloud);
  cout << "Got cloud of size " << cloud.points.size();

  for (unsigned i=0; i<cloud.points.size(); i++)
  {
    cloud.points[i].x -= 19.0;
    cloud.points[i].y -= 20.0;
  }
  
  pcl::io::savePCDFileASCII(filename, cloud);
  cout << "Saved cloud to " << filename;
  
  return 0;
  
}
