/*
 * minioccupancygrid.hh
 * Mac Mason <mmason@willowgarage.com>
 *
 * A (miniature) occupancy grid implementation. Vitally, assumes that all
 * coordinates are positive. Watch out for that.
 *
 * get and set work in meters.
 */

#pragma once
#include <boost/noncopyable.hpp>
#include <utility>
#include <vector>

namespace semanticmodel
{
  class MiniatureOccupancyGrid : public boost::noncopyable
  {
    public:
      // Your choices. Like I said, miniature. Everything defaults to "empty".
      enum Occupancy { EMPTY, OCCUPIED };

      // Starts out empty. Gets bigger automatically. scale is expressed in
      // meters-per-cell. You probably want 0.01, or 0.05, or something like
      // that. 
      MiniatureOccupancyGrid(double scale = 0.05);

      // Get the value at (x, y), measured in meters. Defaults to "empty",
      // should that not exist.
      Occupancy get(double x, double y) const;

      // Set (x, y) (again in meters), to the given value. The underlying grid
      // will be expanded to make room for this.
      void set(double x, double y, Occupancy val);

      std::pair<int, int> sizes() const;

      // 2D ragged array. indexed as [x][y], after coordinate discretization.
      // Making these public makes me cry, but I'm in a hurry.
      std::vector < std::vector<Occupancy> > grid;
      double _scale;

      // Convert from world coordinates ((x, y), in meters) to grid indices
      // ((a, b), in indices). a and b are output arguments.
      void round_coordinates(double x, double y, int& a, int& b) const;
  };
}
