#!/usr/bin/env python
"""
bags_to_database.py
Mac Mason <mmason@willowgarage.com>

We collect bag files; we want to process them into a database. So, this does
that.
"""

import os
import sys
import signal
import time
import subprocess

def handler(signum, frame):
  global RL
  global RB

  if RL:
    RL.kill()
  if RB:
    RB.kill()

signal.signal(signal.SIGTERM, handler)

M = "** We assume that playbag.launch will run, and record to the database\n"\
    "** you want it to. Starting the roscore, etc. that you need for that\n"\
    "** to work is your problem. Enjoy!\n"

print M

if len(sys.argv) < 2:
  print "Usage: bags_to_database.py <one or more bagfiles>"
  sys.exit(1)

# Make sure you've given us only valid files.
for filename in sys.argv[1:]:
  try:
    os.stat(filename)
  except OSError:
    print "File '" + filename + "' does not exist!"


# Here's the deal. We want to fire up playbag.launch, and then rosrun
# semanticmodel playbag.sh <foo>. Once the call to playbag finishes, we want
# to kill the roslaunch, restart the roslaunch, and start the next playbag.

for bagfile in sys.argv[1:]:
  global RL
  global RB
  # Start the roslaunch.
  RL = subprocess.Popen(['roslaunch', 
                         'semanticmodel', 
                         'go.launch',
                         'gui:=false', 'playbag:=true'],  # No gui, please.
                        stdin=None, stdout=None, stderr=None,
                        env=os.environ)

  print "launched for", bagfile

  # Give it a moment to get all sorted out.
  time.sleep(15.0)

  # Now, start the rosbag.
  RB = subprocess.check_call(['rosrun', 
                              'semanticmodel', 
                              'playbag.sh', 
                              bagfile],
                              stdin=None, stdout=None, stderr=None,
                              env=os.environ)

  print "played for", bagfile

  time.sleep(15)  # Make sure we've caught up.

  # check_call waits until this finishes.
  RL.terminate() # No more roslaunch.

  # Give it more time to get all sorted out.
  time.sleep(15.0)

  # and loop!

