#!/usr/bin/env python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of the Willow Garage nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author: Bhaskara Marthi
#
# Downsample bags

import roslib; roslib.load_manifest('semanticmodel')
import rospy
import rosbag as rb
import optparse as op
import sys

def main():
    parser = op.OptionParser()
    parser.add_option("-t", "--topic", dest="topic")
    parser.add_option("-r", "--rate", dest="rate", type="float")
    parser.add_option("-v", action="store_true", dest="verbose", default=False)
    options, args = parser.parse_args()

    ds_topic = options.topic
    assert options.rate > 0
    d = 1/options.rate
    if not ds_topic.startswith('/'):
        ds_topic = '/'+ds_topic
        

    inbag = args[0]
    last = None
    last_print = None
    t0 = None
    with rb.Bag(args[1], 'w') as outbag:
        for topic, msg, time in rb.Bag(inbag).read_messages(raw=True):
            store = True
            ts = time.to_sec()
            if not t0:
                t0 = ts
            if not last_print or last_print < ts-1:
                print("Time: {0}".format(ts-t0))
                last_print = ts
            if topic == ds_topic:
                if last and last>ts-d:
                    store = False
                    if options.verbose:
                        print("Ignoring message at ", ts)
                else:
                    last = ts
                    if options.verbose:
                        print("Storing message at ", last)
            if store:
                outbag.write(topic, msg, t=time, raw=True)
                if options.verbose:
                    print("Stored message on {0} for time {1}".format(topic, time))
            
        
        
        
    
    


if __name__ == "__main__":
    main()
