#! /usr/bin/python
#***********************************************************
#* Software License Agreement (BSD License)
#*
#*  Copyright (c) 2009, Willow Garage, Inc.
#*  All rights reserved.
#*
#*  Redistribution and use in source and binary forms, with or without
#*  modification, are permitted provided that the following conditions
#*  are met:
#*
#*   * Redistributions of source code must retain the above copyright
#*     notice, this list of conditions and the following disclaimer.
#*   * Redistributions in binary form must reproduce the above
#*     copyright notice, this list of conditions and the following
#*     disclaimer in the documentation and/or other materials provided
#*     with the distribution.
#*   * Neither the name of Willow Garage, Inc. nor the names of its
#*     contributors may be used to endorse or promote products derived
#*     from this software without specific prior written permission.
#*
#*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#*  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#*  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#*  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#*  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#*  POSSIBILITY OF SUCH DAMAGE.
#* 
#* Author: Eitan Marder-Eppstein
#***********************************************************
PKG='semanticmodel'
import roslib
roslib.load_manifest(PKG)
import rospy
import time
import actionlib

from move_base_msgs.msg import *
from geometry_msgs.msg import *
from roslib.msg import *

# Original goals; who knows where from.
#goals = [
#    [(19.380, 24.061, 0.000), (0.000, 0.000, 0.953, -0.304)],
#    [(27.193, 15.448, 0.000), (0.000, 0.000, 0.526, 0.851)],
#    [(29.872, 21.884, 0.000), (0.000, 0.000, 0.979, 0.206)],
#    [(18.632, 26.155, 0.000), (0.000, 0.000, 0.807, 0.590)],
#    [(20.772, 38.251, 0.000), (0.000, 0.000, 0.980, 0.201)],
#    [(5.184, 22.094, 0.000), (0.000, 0.000, 0.559, 0.829)],
#    [(17.346, 54.900, 0.000), (0.000, 0.000, 0.160, 0.987)],
#    [(25.170, 52.124, 0.000), (0.000, 0.000, -0.099, 0.995)],
#    [(20.409, 38.265, 0.000), (0.000, 0.000, -0.678, 0.735)],
#    [(24.109, 20.406, 0.000), (0.000, 0.000, 0.973, 0.230)]
#]

# Mac's goals.
#goals = [
#    [(20.11, 25.77, 0.0), (0.0, 0.0, -0.83, 0.55)],
#    [(17.85, 19.21, 0.0), (0.0, 0.0, -0.19, 0.98)],
#    [(27.06, 15.57, 0.0), (0.0, 0.0, 0.56, 0.82)],
#    [(35.37, 20.52, 0.0), (0, 0, -0.96, -0.25)],
#    [(31.26, 25.06, 0.0), (0, 0, -0.829, 0.55)],
#    [(19.78, 33.32, 0.0), (0, 0, 0.6, 0.79)],
#    [(20.76, 37,84, 0.0), (0, 0, 0.98, 0.17)],
#    [(19.22, 24.29, 0.0), (0, 0, -0.71, 0.69)]
#]

# Mac's (long tour) goals.
goals = [
    [(18.1692905426, 19.1994800568, 0.0), 
     (0.0, 0.0, -0.192855206976, 0.98122722605)],
    [(29.8882846832, 21.8147888184, 0.0), 
     (0.0, 0.0, 0.556712602762, 0.830705169074)],
    [(36.4701004028, 22.6947193146, 0.0),
     (0.0, 0.0, -0.838297314816, 0.545213363714)],
    [(34.0851974487, 16.0869045258, 0.0),
     (0.0, 0.0, 0.904685123893, 0.426080774744)],
    [(28.3673648834, 10.1040353775, 0.0),
     (0.0, 0.0, 0.976806145604, 0.214125556417)],
    [(28.1669883728, 17.825050354, 0.0),
     (0.0, 0.0, 0.527215004905, 0.84973192161)],
    [(23.5591754913, 24.3424053192, 0.0),
     (0.0, 0.0, 0.980031762909, 0.19884100103)],
    [(19.0632476807, 33.7512588501, 0.0),
     (0.0, 0.0, 0.250532110313, 0.968108290277)],
    [(8.69736289978, 30.8772182465, 0.0),
     (0.0, 0.0, -0.834096085225, 0.551619180787)],
    [(7.90707015991, 31.473531723, 0.0),
     (0.0, 0.0, 0.652970028173, 0.757383748378)],
    [(17.3875350952, 55.0858535767, 0.0),
     (0.0, 0.0, 0.0781252043804, 0.996943555293)],
    [(25.609790802, 52.1704063416, 0.0),
     (0.0, 0.0, -0.831385731323, 0.555695749266)],
    [(20.7248096466, 38.6204910278, 0.0),
     (0.0, 0.0, -0.727519456268, 0.686087050418)],
    [(17.118265152, 17.8405857086, 0.0),
     (0.0, 0.0, 0.418973493025, 0.907998464835)]
]


def mb_goal(goal):
    return MoveBaseGoal(PoseStamped(Header(frame_id = 'map'),
                                      Pose(Point(goal[0][0], goal[0][1],
                                                 goal[0][2]),
                                           Quaternion(goal[1][0], goal[1][1],
                                                      goal[1][2], goal[1][3]))))
if __name__ == '__main__':
    rospy.init_node('willow_tour_node')
    client = actionlib.SimpleActionClient('move_base', MoveBaseAction)
    rospy.loginfo("Waiting for move_base")
    client.wait_for_server()
    rospy.loginfo("Sending goals")

    for goal in goals:
        mb = mb_goal(goal)
        rospy.loginfo("Sending goal %s" % goal)
        client.send_goal(mb)
        client.wait_for_result()

    rospy.loginfo('Done with tour')

