#!/usr/bin/env python

# Software License Agreement (BSD License)
#
# Copyright (c) 2008, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of Willow Garage, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: Bhaskara Marthi

# Node that generates a unique name for the collection namespace used in the
# world model and inserts some metadata into it.

import roslib
roslib.load_manifest('semanticmodel')

import rospy
import pymongo as pm
import time
import std_srvs.srv
import std_msgs.msg
import semanticmodel.srv as sms
import semanticmodel.db as sdb

# Look in the db 'runs' collection for the largest existing run id
# and create namespace with an id one larger
def update_collection_namespace():
    global COLLECTION_NAMESPACE
    global BAGFILE
    db = CONN[DB_NAME]
    next_id = sdb.generate_run_id(db)
    COLLECTION_NAMESPACE = "run{0}".format(next_id)
    while not rospy.is_shutdown():
        t = rospy.Time.now().to_sec()
        if t>0.0:
            rospy.loginfo("Setting run namespace to {0}".format(next_id))
            db.runs.insert({'id': next_id, 'bag_file': BAGFILE,
                         'creation_time': rospy.Time.now().to_sec()})
            break
        else:
            rospy.loginfo("Waiting till ros time is > 0")
            time.sleep(2.0)

def get_collection_namespace(req):
    return {'db_name': DB_NAME, 'collection_namespace': COLLECTION_NAMESPACE}
       

rospy.init_node('sync_db_name')

HOST = rospy.get_param('warehouse_host', 'bab.willowgarage.com')
PORT = rospy.get_param('warehouse_port', 27017)
DB_NAME = rospy.get_param('semantic_db_name', 'semantic_world_model')


rospy.loginfo("Starting up sync_db_name.  Expect db server at {0}:{1} and db name {2}".
    format(HOST, PORT, DB_NAME))
COLLECTION_NAMESPACE = None
CONN = None


while not rospy.is_shutdown():
    try:
        BAGFILE = rospy.get_param('current_bagfile')
        break
    except:
        rospy.logwarn("Waiting for there to exist a parameter called current_bagfile")
        time.sleep(2.0)

rospy.loginfo("Found current_bagfile={0}".format(BAGFILE))


rospy.loginfo("Attempting to connect to db at {0}:{1}".
              format(HOST, PORT))

while not rospy.is_shutdown():
    try:
        CONN=pm.Connection(host=HOST, port=PORT)
        rospy.loginfo("Connected to db")
        break
    except Exception as e:
        rospy.logwarn("Failed to connect to MongoDB at {1}:{2}; retrying.  Exception was: {0}".
                      format(e, HOST, PORT))
        rospy.sleep(2.0)
        
update_collection_namespace()
get_name_srv = rospy.Service('get_collection_namespace', 
                             sms.GetCollectionNamespace, 
                             get_collection_namespace)
rospy.spin()    
