/*
 * miniatureoccupancygrid.cc
 * Mac Mason <mmason@willowgarage.com>
 *
 * Implementation. See miniatureoccupancygrid.hh.
 */

#include <cassert>
#include "semanticmodel/miniatureoccupancygrid.hh"

namespace semanticmodel
{
  MiniatureOccupancyGrid::MiniatureOccupancyGrid(double scale)
    : grid(), _scale(scale)
  {
  }

  // No destructor needed.
  
  MiniatureOccupancyGrid::Occupancy MiniatureOccupancyGrid::get(double x, 
                                                                double y) const
  {
    int a, b;
    round_coordinates(x, y, a, b);

    assert (x >= 0);
    assert (y >= 0);

    if (x >= grid.size())
      return MiniatureOccupancyGrid::EMPTY;  // Default.
    else if (y >= grid.at(x).size())
      return MiniatureOccupancyGrid::EMPTY;  // Ditto.
    else
      return grid.at(x).at(y);
  }

  void MiniatureOccupancyGrid::set(double x, double y, 
                                   MiniatureOccupancyGrid::Occupancy val) 
  {
    while (grid.size() <= x)
      grid.push_back(std::vector<MiniatureOccupancyGrid::Occupancy>());

    while (grid.at(x).size() <= y)
      grid.at(x).push_back(MiniatureOccupancyGrid::EMPTY);

    grid.at(x).at(y) = val;
  }

  void MiniatureOccupancyGrid::round_coordinates(double x, double y, 
                                                 int& a, int& b) const
  {
    // First, we need to convert x and y into grid units.
    double nx = x / _scale;
    double ny = y / _scale;

    a = (int)nx;
    b = (int)ny;
  }
}
